--- kernel-source-2.4.21/arch/i386/kernel/io_apic.c.up-ioapic-resend-irq	2003-07-20 23:26:40 +0400
+++ kernel-source-2.4.21/arch/i386/kernel/io_apic.c	2003-07-21 17:59:35 +0400
@@ -1675,6 +1675,58 @@
 	print_IO_APIC();
 }
 
+#ifdef CONFIG_X86_UP_IOAPIC
+
+/*
+ * Seems that the uniprocessor IO-APIC support needs working hw_resend_irq(),
+ * otherwise edge-triggered IO-APIC IRQs may be lost if disable_irq() is used
+ * (happens with the IDE driver).
+ *
+ * As a temporary solution, these functions are copied from smp.c.
+ */
+
+/* MUST have send_IPI_self() prototype - it is FASTCALL */
+#include <asm/hw_irq.h>
+
+static inline int __prepare_ICR (unsigned int shortcut, int vector)
+{
+	return APIC_DM_FIXED | shortcut | vector | INT_DEST_ADDR_MODE;
+}
+
+static inline void __send_IPI_shortcut(unsigned int shortcut, int vector)
+{
+	/*
+	 * Subtle. In the case of the 'never do double writes' workaround
+	 * we have to lock out interrupts to be safe.  As we don't care
+	 * of the value read we use an atomic rmw access to avoid costly
+	 * cli/sti.  Otherwise we use an even cheaper single atomic write
+	 * to the APIC.
+	 */
+	unsigned int cfg;
+
+	/*
+	 * Wait for idle.
+	 */
+	apic_wait_icr_idle();
+
+	/*
+	 * No need to touch the target chip field
+	 */
+	cfg = __prepare_ICR(shortcut, vector);
+
+	/*
+	 * Send the IPI. The write to APIC_ICR fires this off.
+	 */
+	apic_write_around(APIC_ICR, cfg);
+}
+
+void send_IPI_self(int vector)
+{
+	__send_IPI_shortcut(APIC_DEST_SELF, vector);
+}
+
+#endif /* CONFIG_X86_UP_IOAPIC */
+
 
 /* --------------------------------------------------------------------------
                           ACPI-based IOAPIC Configuration
--- kernel-source-2.4.21/include/asm-i386/hw_irq.h.up-ioapic-resend-irq	2003-07-20 19:51:14 +0400
+++ kernel-source-2.4.21/include/asm-i386/hw_irq.h	2003-07-21 17:36:03 +0400
@@ -213,7 +213,7 @@
 	atomic_inc((atomic_t *)&prof_buffer[eip]);
 }
 
-#ifdef CONFIG_SMP /*more of this file should probably be ifdefed SMP */
+#if defined(CONFIG_SMP) || defined(CONFIG_X86_UP_IOAPIC) /*more of this file should probably be ifdefed SMP */
 static inline void hw_resend_irq(struct hw_interrupt_type *h, unsigned int i) {
 	if (IO_APIC_IRQ(i))
 		send_IPI_self(IO_APIC_VECTOR(i));
